/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/
package net.moioli.drawables;

import java.lang.Double;

import net.moioli.chart.DrawableI;
import net.moioli.chart.GraphicsI;
import net.moioli.chart.Scale;

/**
 * A general implementation of the DrawableI and FunctionI interfaces. Includes
 * empirical minY() and maxY() methods.
 * 
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public abstract class AbstractDrawableFunction implements DrawableI, FunctionI {
    /**
     * Parameter for the emprical minY/maxY detection. The function will be
     * re-calculated every (maxX()-minX())/ACCURACY wide step.
     */
    private static final double ACCURACY = 50;

    /** Default constructor, does really nothing. */
    public AbstractDrawableFunction() {
        super();
    }

    /**
     * @see FunctionI#f(double)
     */
    public abstract double f(double x);

    /**
     * Returns this.f(x), or NaN if an exception occured while computing.
     * 
     * @param x
     *            the abscissa
     * @return internalFunction(x) or NaN
     */
    private double safeF(double x) {
        double result;
        try {
            result =  this.f(x);
        } catch (ArithmeticException e) {
            result =  Double.NaN;
        }
        return result;
    }

    /**
     * Returns the point in which the function is minimum with an empirical
     * algorithm. While this method is accurate enough in most cases, it should
     * be overridden whenever possible.
     * 
     * @param minX
     *            the minimum x value to be represented
     * @param maxX
     *            the maximum x value to be represented
     * @return the abcissa in which the function has an absolute minimum
     */
    public double getMinimum(double minX, double maxX) {
        double step = (maxX - minX) / ACCURACY;
        double acc = minX;
        double minValue = Double.POSITIVE_INFINITY;
        double result = minX;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeF(acc);
            if (value < minValue) {
                minValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * Returns the point in which the function is maximum with an empirical
     * algorithm. While this method is accurate enough in most cases, it should
     * be overridden whenever possible.
     * 
     * @param minX
     *            the minimum x value to be represented
     * @param maxX
     *            the maximum x value to be represented
     * @return the abcissa in which the function has an absolute maximum
     */
    public double getMaximum(double minX, double maxX) {
        double step = (maxX - minX) / ACCURACY;
        double acc = minX;
        double maxValue = Double.NEGATIVE_INFINITY;
        double result = minX;
        double value;
        for (int i = 0; i < ACCURACY; i++) {
            value = this.safeF(acc);
            if (value > maxValue) {
                maxValue = value;
                result = acc;
            }
            acc += step;
        }
        return result;
    }

    /**
     * @see net.moioli.chart.DrawableI#minY(double, double)
     */
    public double minY(double minX, double maxX) {
        return this.safeF(this.getMinimum(minX, maxX));
    }

    /**
     * @see net.moioli.chart.DrawableI#maxY(double, double)
     */
    public double maxY(double minX, double maxX) {
        return this.safeF(this.getMaximum(minX, maxX));
    }

    /**
     * @see net.moioli.chart.DrawableI#drawMe(GraphicsI, Scale, Object)
     */
    public void drawMe(GraphicsI g, Scale s, Object c) {
        double minX = s.getMinX();
        double maxX = s.getMaxX();
        double minY = s.getMinY();
        double maxY = s.getMaxY();
        double step = s.getStep();
        double j;
        g.setColor(c);

        double thisX = minX;
        double previousX = minX;
        double thisY = 0;
        double previousY = 0;

        for (j = minX; j <= maxX; j += step) {
            previousY = safeF(j);
            if (!Double.isNaN(previousY)) {
                previousX = j;
                break;
            }
        }

        // j is already inited
        for (; j <= maxX; j += step) {
            thisY = safeF(j);
            if (!Double.isNaN(thisY)) {
                thisX = j;
                if (thisY > minY && thisY < maxY) {
                    g.drawLine(s.getX(previousX), s.getY(previousY), s
                            .getX(thisX), s.getY(thisY));
                }
                previousX = thisX;
                previousY = thisY;
            }
        }
    }
}