/***************************************************************************
 * MoioChart Library - graphs and charts made easy                         *
 * Copyright (C) 2005 by Silvio Moioli                                     *
 * silvio@moioli.net, http://www.moioli.net                                *  
 *                                                                         *
 * This program is free software; you can redistribute it and/or           *
 * modify it under the terms of the GNU General Public License             *
 * as published by the Free Software Foundation; either version 2          *
 * of the License, or (at your option) any later version.                  *
 *                                                                         *
 * This program is distributed in the hope that it will be useful,         *
 * but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 * GNU General Public License for more details.                            *
 *                                                                         *
 * You should have received a copy of the GNU General Public License       *
 * along with this program; if not, write to the Free Software             *
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,              *
 * MA  02110-1301, USA.                                                    *
 ***************************************************************************/

package net.moioli.drawables;

import org.nfunk.jep.JEP;
import org.nfunk.jep.ParseException;

/**
 * This class provides a quick way to implement mathematical curves: parsing
 * them from strings. Just pass a string like "x^2+sin(x)" and "3y" to the
 * constructor and you'll get a drawable curve to add to your DefaultChart. <br />
 * The parsing is done with the JEP parser, so strings should follow JEP's
 * syntax.
 * 
 * @see org.nfunk.jep.JEP
 * @author Silvio Moioli
 * @version 2005/10/5
 */
public class ParsedPlaneCurve extends AbstractPlaneCurve {
    /** Parser object for the x function */
    private JEP xParser;

    /** Parser object for the x function */
    private JEP yParser;

    /** String representing the x function */
    private String xFunction;

    /** String representing the y function */
    private String yFunction;

    /**
     * Constructs a new function with the given strings (any point in this
     * function has coordinates (x(t),y(t)). The string should follow JEP's
     * syntax.
     * 
     * @see org.nfunk.jep.JEP
     * @param xFunction
     *            the string to be parsed corresponting to x(t)
     * @param yFunction
     *            the string to be parsed corresponting to y(t)
     * @param maxT
     *            the maximum parameter value
     * @throws ParseException
     *             if the string cannot be parsed by JEP
     */
    public ParsedPlaneCurve(String xFunction, String yFunction, double maxT)
            throws ParseException {
        super(maxT);
        this.xParser = new JEP();
        this.xParser.addStandardFunctions();
        this.xParser.addStandardConstants();
        this.xParser.setImplicitMul(true);
        this.xParser.addVariable("t", 0);
        this.xParser.parseExpression(xFunction);
        if (this.xParser.hasError()) {
            throw new ParseException("Couldn't parse: " + xFunction);
        }
        this.xFunction = xFunction;

        // Same for y
        this.yParser = new JEP();
        this.yParser.addStandardFunctions();
        this.yParser.addStandardConstants();
        this.yParser.setImplicitMul(true);
        this.yParser.addVariable("t", 0);
        this.yParser.parseExpression(yFunction);
        if (this.yParser.hasError()) {
            throw new ParseException("Couldn't parse: " + yFunction);
        }
        this.yFunction = yFunction;
    }

    /**
     * @see net.moioli.drawables.PlaneCurveI#x(double)
     */
    public double x(double t) {
        this.xParser.addVariable("t", t);
        return this.xParser.getValue();
    }

    /**
     * @see net.moioli.drawables.PlaneCurveI#y(double)
     */
    public double y(double t) {
        this.yParser.addVariable("t", t);
        return this.yParser.getValue();
    }

    /**
     * Returns a string describing this function.
     * 
     * @return a string describing this function
     */
    public String toString() {
        return "(" + this.xFunction + ";" + this.yFunction + ")";
    }
}
